﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Infragistics.Web.Mvc;
using Sample.Models;
using Sample.Data;
using Sample.Data.BusinessObjects;

namespace Sample.Controllers
{
    public class GridModelController : Controller
    {

        public ActionResult DefinedColumns()
        {
            ViewModel viewModel = new ViewModel();

            viewModel.Grid = GetGridModel();
            viewModel.Grid.DataSource = null;

            return View(viewModel);
        }

        public ActionResult Index()
        {
            return View();
        }

        // [BT] This is only a stub - it's not meant to persist any changes - only to return the Success response.
        public ActionResult EditingSaveChanges()
        {
            ViewData["GenerateCompactJSONResponse"] = false;

            JsonResult result = new JsonResult();
            Dictionary<string, bool> response = new Dictionary<string, bool>();
            response.Add("Success", true);
            result.Data = response;
            return result;
        }

        private GridModel GetGridModel()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = false;
            gridModel.AutoGenerateLayouts = false;
            gridModel.AutoCommit = true;
            gridModel.LoadOnDemand = true;
            gridModel.PrimaryKey = "ProjectId";

            gridModel.Columns = new List<GridColumn>();
            gridModel.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "100px"));
            gridModel.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            gridModel.Columns.Add(new GridColumn("Start Date", "StartDate", "date", "160px"));
            gridModel.Columns.Add(new GridColumn("End Date", "EndDate", "date", "160px"));
            gridModel.Columns.Add(new GridColumn("Is Postponed", "IsPostponed", "bool", "110px"));

            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindProjects");
            gridModel.UpdateUrl = Url.Action("EditingSaveChanges");

            GridUpdating updating = new GridUpdating();
            updating.EditMode = GridEditMode.Row;
            gridModel.Features.Add(updating);

            GridGroupBy groupby = new GridGroupBy();
            groupby.Type = OpType.Remote;
            gridModel.Features.Add(groupby);

            GridFiltering filtering = new GridFiltering();
            filtering.Type = OpType.Remote;
            gridModel.Features.Add(filtering);



            return gridModel;
        }

        #region Data Access

        private IQueryable<MyProject> GetProjectData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from p in entities.Projects
                        orderby p.ProjectID
                        select new MyProject
                        {
                            ProjectId = p.ProjectID,
                            Name = p.Name,
                            StartDate = p.StartDate,
                            EndDate = p.EndDate,
                            IsPostponed = p.IsPostponed,
                            Developers = from d in p.Developers
                                         orderby d.DeveloperID
                                         select new MyDeveloper
                                         {
                                             DeveloperId = d.DeveloperID,
                                             ProjectId = d.ProjectID,
                                             Name = d.Name,
                                             Position = d.Position,
                                             HireDate = d.HireDate,
                                             Salary = d.Salary,
                                             VirtualMachines = from v in d.VirtualMachines
                                                               orderby v.VirtualMachineID
                                                               select new MyVirtualMachine
                                                               {
                                                                   VirtualMachineId = v.VirtualMachineID,
                                                                   DeveloperId = v.DeveloperID,
                                                                   OperatingSystem = v.OperatingSystem,
                                                                   DelegatedSpace = v.DelegatedSpace,
                                                                   ExpirationDate = v.ExpirationDate
                                                               }
                                         },
                            Scrums = from s in p.Scrums
                                     orderby s.MeetingID
                                     select new MyScrum
                                     {
                                         ProjectId = s.ProjectID,
                                         MeetingId = s.MeetingID,
                                         OccurredOn = s.OccurredOn,
                                         ScrumMaster = s.ScrumMaster,
                                         Notes = s.Notes,
                                         Priority = s.Priority,
                                         AreAllRequired = s.AreAllRequired,
                                         ScrumMeetings = new MyScrumMeeting()
                                         {
                                             MeetingId = s.ScrumMeetings.MeetingID,
                                             Name = s.ScrumMeetings.Name,
                                             Description = s.ScrumMeetings.Description,
                                             MaxDuration = s.ScrumMeetings.MaxDuration
                                         }
                                     },
                            Tasks = from t in p.Tasks
                                    orderby t.TaskID
                                    select new MyTask
                                    {
                                        TaskId = t.TaskID,
                                        ProjectId = t.ProjectID,
                                        Name = t.Name,
                                        ParentTaskId = t.ParentTaskID,
                                        IsCompleted = t.IsCompleted,
                                        Checkins = from c in t.Checkins
                                                   orderby c.CheckinID
                                                   select new MyCheckin
                                                   {
                                                       CheckinId = c.CheckinID,
                                                       TaskId = c.TaskID,
                                                       CheckinDate = c.CheckinDate,
                                                       Files = from f in c.Files
                                                               orderby f.FileID
                                                               select new MyFile
                                                               {
                                                                   FileId = f.FileID,
                                                                   CheckinId = f.CheckinID,
                                                                   Name = f.Name,
                                                                   Size = f.Size
                                                               }
                                                   }
                                    }
                        };
            return query;
        }

        private IEnumerable<MyDeveloper> GetDevelopersData()
        {
            DevelopmentDBEntities entities = new DevelopmentDBEntities();
            var query = from d in entities.Developers
                        orderby d.DeveloperID
                        select new MyDeveloper
                        {
                            DeveloperId = d.DeveloperID,
                            ProjectId = d.ProjectID,
                            Name = d.Name,
                            Position = d.Position,
                            HireDate = d.HireDate,
                            Salary = d.Salary,
                            VirtualMachines = from v in d.VirtualMachines
                                              orderby v.VirtualMachineID
                                              select new MyVirtualMachine
                                              {
                                                  VirtualMachineId = v.VirtualMachineID,
                                                  DeveloperId = v.DeveloperID,
                                                  OperatingSystem = v.OperatingSystem,
                                                  DelegatedSpace = v.DelegatedSpace,
                                                  ExpirationDate = v.ExpirationDate
                                              }
                        };
            return query;
        }

        #endregion // Data Access

        #region DataSourceUrls

        public JsonResult BindAutoGeneratedProjects()
        {
            GridModel gridModel = new GridModel();
            gridModel.AutoGenerateColumns = true;
            gridModel.AutoGenerateLayouts = true;
            gridModel.DataSource = GetProjectData();
            gridModel.DataSourceUrl = Url.Action("BindAutoGeneratedProjects");
            gridModel.LoadOnDemand = false;

            ViewModel viewModel = new ViewModel();

            GridSorting rootSorting = new GridSorting();
            rootSorting.Type = OpType.Remote;
            rootSorting.Mode = SortingMode.Multiple;
            rootSorting.Inherit = false;
            gridModel.Features.Add(rootSorting);

            GridPaging rootPaging = new GridPaging();
            rootPaging.Type = OpType.Remote;
            rootPaging.Inherit = false;
            gridModel.Features.Add(rootPaging);

            viewModel.Grid = gridModel;
            return gridModel.GetData();
        }

        public JsonResult BindProjects()
        {
            GridModel gridModel = GetGridModel();
            return gridModel.GetData();
        }

        /// <summary>
        /// Use this method if you wish to bind the grid to the Developers table
        /// </summary>
        /// <returns></returns>
        public JsonResult BindDevelopers()
        {
            GridModel gridModel = GetGridModel();
            gridModel.DataSource = GetDevelopersData();
            return gridModel.GetData();
        }
        #endregion // DataSourceUrls

        #region Layouts
        private GridColumnLayoutModel GetDevelopersLayout()
        {
            GridColumnLayoutModel layout = new GridColumnLayoutModel();
            layout.Key = "Developers";
            layout.PrimaryKey = "DeveloperId";
            layout.ForeignKey = "ProjectId";
            layout.Caption = "Developers";
            layout.Columns = new List<GridColumn>();
            layout.Columns.Add(new GridColumn("Developer Id", "DeveloperId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Project Id", "ProjectId", "number", "200px"));
            layout.Columns.Add(new GridColumn("Name", "Name", "string", "200px"));
            layout.Columns.Add(new GridColumn("Positon", "Position", "string", "200px"));
            layout.Columns.Add(new GridColumn("Hire Date", "HireDate", "date", "200px"));
            layout.Columns.Add(new GridColumn("Salary", "Salary", "number", "200px"));

            layout.DataSourceUrl = Url.Action("BindDevelopers");

            return layout;
        }
        #endregion // Layouts
    }
}
